package skyjed;

import java.io.*;
import java.util.Arrays;

import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import skyjed.gui.MainWindow;
import skyjed.save.SaveFile;
import skyjed.util.ExceptionHandler;

public class SkyJEd {

    /**
     * Launch the application.
     */
    public static void main(String[] args) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                try {
                    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                    new MainWindow();
                } catch (Exception e) {
                    ExceptionHandler.handleException(e, null);
                }
            }
        });
    }

    public static SaveFile openFile(File file) throws IOException {
        long length = file.length();
        checkFileLength(length, true);
        byte[] rawbuf = new byte[(int) length];
        try (DataInputStream fin = new DataInputStream(new FileInputStream(file))) {
            fin.readFully(rawbuf);
        }
        return new SaveFile(rawbuf);
    }

    public static void saveFile(File file, SaveFile save) throws IOException {
        try (FileOutputStream fout = new FileOutputStream(file)) {
            fout.write(save.toByteA());
        }
    }

    public static void truncateFile(File file) throws IOException {
        long length = file.length();
        checkFileLength(length, false);
        byte[] rawbuf = new byte[SaveFile.MIN_FILE_LENGTH];
        try (DataInputStream fin = new DataInputStream(new FileInputStream(file))) {
            fin.readFully(rawbuf);
        }
        try (FileOutputStream fout = new FileOutputStream(file)) {
            fout.write(rawbuf);
        }
    }

    private static void checkFileLength(long length, boolean upper) throws IOException {
        if(length < SaveFile.MIN_FILE_LENGTH) {
            throw new IOException("File is too short for a pkmn md sky save");
        }
        if(upper && length >= 4*1024*1024) { // 4MB
            throw new IOException(
                    "File is way too big for a save file.\n" +
                    "If you're sure you may want to truncate this file first.");
        }
    }

    public static boolean testLoadSave(File file) throws IOException {
        long length = file.length();
        checkFileLength(length, true);
        byte[] rawbuf = new byte[SaveFile.MIN_FILE_LENGTH];
        try (DataInputStream fin = new DataInputStream(new FileInputStream(file))) {
            fin.readFully(rawbuf);
        }

        byte[] tbuf = Arrays.copyOf(rawbuf, rawbuf.length);
        SaveFile tsave = new SaveFile(rawbuf);

        // DEBUG BEGIN
        //skyjed.buffer.BooleanBuffer buf = tsave.slot1.buf;
        //skyjed.buffer.BooleanBuffer qbuf = tsave.qslot.buf;
        /*
        buf.seek(0x8A3F*8+2);
        for(int i=0;i<600;i++) if(buf.get()) System.out.println(skyjed.resources.SkyPokemonRes.getPkmnName(i));
        buf.seek(0x8A8A*8+2); // no-op
        for(int i=0;i<553;i++) if(buf.get()) System.out.println(skyjed.resources.SkyPokemonRes.getPkmnName(i));
         */
        /*
        System.out.println("searching");
        for(int i=0; i<buf.size()-546; i++) {
            buf.seek(i);
            if(buf.getInt(17) == 1465) {
                System.out.println(String.format("0x%04x*8+%d", i/8, i%8));
            }
        }
         */
        /*
        buf.seek(0x9958*8+0);
        System.out.println(buf.getInt(32));
         */
        //for(int p=0;p<4;p++) tsave.slot1.activePkmn.pkmns[p].dump();
        /*
        qbuf.seek(0x1C170*8-102400*8);
        for(int p=0;p<20;p++) {
            for(int i=0;i<429;i++) { System.out.print(String.format("%3d ", qbuf.getInt(8))); if(i%16==15) System.out.print("|"); }
            System.out.println();
        }
         */
        //for(int p=0;p<20;p++) tsave.qslot.pkmnQStorage.pkmns[p].dump();
        //tsave.slot1.adventureLog.dump();
        // DEBUG END

        return Arrays.equals(tbuf, tsave.toByteA());
    }

}
